// SPDX-License-Identifier: MIT

#ifndef QUANTIZR_H
#define QUANTIZR_H


#define QUANTIZR_MAJOR 1
#define QUANTIZR_MINOR 4
#define QUANTIZR_PATCH 2


#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

typedef enum {
  QUANTIZR_OK = 0,
  QUANTIZR_VALUE_OUT_OF_RANGE = 100,
  QUANTIZR_BUFFER_TOO_SMALL = 1,
} QuantizrError;

/**
 * Color histogram
 */
typedef struct QuantizrHistogram QuantizrHistogram;

/**
 * Image reference containing pixel data and dimensions info
 */
typedef struct QuantizrImage QuantizrImage;

/**
 * Quantization options
 */
typedef struct QuantizrOptions QuantizrOptions;

typedef struct QuantizrResult QuantizrResult;

/**
 * RGBA color
 */
typedef struct {
  uint8_t r;
  uint8_t g;
  uint8_t b;
  uint8_t a;
} QuantizrColor;

/**
 * Color palette
 */
typedef struct {
  /**
   * The number of colors in the palette
   */
  uint32_t count;
  /**
   * The palette colors
   */
  QuantizrColor entries[256];
} QuantizrPalette;

QuantizrOptions *quantizr_new_options(void);

QuantizrError quantizr_set_max_colors(QuantizrOptions *options, int32_t colors);

QuantizrImage *quantizr_create_image_rgba(const uint8_t *data, int32_t width, int32_t height);

QuantizrHistogram *quantizr_create_histogram(void);

QuantizrError quantizr_histogram_add_image(QuantizrHistogram *hist, const QuantizrImage *image);

QuantizrResult *quantizr_quantize(const QuantizrImage *image, const QuantizrOptions *options);

QuantizrResult *quantizr_quantize_histogram(const QuantizrHistogram *hist,
                                            const QuantizrOptions *options);

QuantizrError quantizr_set_dithering_level(QuantizrResult *result, float dither);

const QuantizrPalette *quantizr_get_palette(const QuantizrResult *result);

float quantizr_get_error(const QuantizrResult *result);

QuantizrError quantizr_remap(const QuantizrResult *result,
                             const QuantizrImage *image,
                             uint8_t *buffer,
                             uintptr_t buffer_size);

void quantizr_free_result(QuantizrResult *result);

void quantizr_free_histogram(QuantizrHistogram *hist);

void quantizr_free_image(QuantizrImage *image);

void quantizr_free_options(QuantizrOptions *options);

#endif /* QUANTIZR_H */
